{$MODE objfpc}

Type
  Float = Double;
  TPoint = Record
    X, Y : Integer;
  End;
  TVector = Record
    X, Y : Float;
  End;
  PNorm = ^TNorm;
  TNorm = Array[0..1023, 0..1023] Of TVector; {16mb}

Var
  N : Integer;
  K : Integer;
  Point : Array[0..1234] Of TPoint;
  Gr : Array[0..1023, 0..1023] Of Integer; {4mb}
  traced : Array[0..1023, 0..1023] Of Boolean; {1mb}
{  Norm : Array[0..1023, 0..1023] Of TVector;} {16mb}
  Norm : PNorm;
  PathLen : Integer;
  Path : Array[0..1234] Of Integer;
  Vis : Array[0..1234] Of Boolean;
  Res : Integer;

Procedure WriteOut;

Var
  F : Text;

Begin
  ASSign(F, 'ZAD6.RES');
  Rewrite(F);
  Writeln(F, Res);
  Close(F);
End;

Procedure Precalc_Norm(A, B : Integer);

Var
  magn : Float;

Begin
{  Writeln(A, ' ', B);}
  Norm^[A, B].X := Point[B].X - Point[A].X;
  Norm^[A, B].Y := Point[B].Y - Point[A].Y;
  magn := Sqrt(Norm^[A, B].X * Norm^[A, B].X + Norm^[A, B].Y * Norm^[A, B].Y);
  Norm^[A, B].X /= magn;
  Norm^[A, B].Y /= magn;
End;

Function DotProduct(V1, V2 : TVector) : Float;

Begin
  DotProduct := V1.X * V2.X + V1.Y * V2.Y;
End;

Function SqDist(X1, Y1, X2, Y2 : Integer) : Integer;

Begin
  SqDist := Sqr(X2 - X1) + Sqr(Y2 - Y1);
End;

Function SqDist(A, B : Integer) : Integer;

Begin
  SqDist := SqDist(Point[A].X, Point[A].Y, Point[B].X, Point[B].Y);
End;

Function Dir(X1, Y1, X2, Y2, X3, Y3 : Integer) : Integer;

Begin
  Dir := (Y2 - Y3) * (X2 - X1) + (Y2 - Y1) * (X3 - X2);
End;

Function Dir(A, B, C : Integer) : Integer;

Begin
  Dir := Dir(Point[A].X, Point[A].Y,
             Point[B].X, Point[B].Y,
             Point[C].X, Point[C].Y);
End;

Procedure ReadIn;

Var
  F : Text;
  I, J : Integer;

Begin
  ASSign(F, 'ZAD6.DAT');
  Reset(F);
  Readln(F, N);
  For I := 1 To N Do
  Begin
    Read(F, Point[I].X, Point[I].Y);
    Read(F, Gr[I, 0]);
    For J := 1 To Gr[I, 0] Do
      Read(F, Gr[I, J]);
    Readln(F);
  End;
  Readln(F, K);
  Close(F);
End;

Function Trace(A, B : Integer) : Integer; {a->b}

Var
  I : Integer;
  positive : Boolean;
  bestp, bestsqdist : Integer;
  newsqdist : Integer;
  bestdp, newdp : Float;

Begin
  positive := False;
  For I := 1 To Gr[B, 0] Do
  Begin
    If Dir(A, B, Gr[B, I]) > 0 Then
    Begin
      positive := True;
      Break;
    End;
  End;
  If positive Then
  Begin
    bestdp := 200;
    bestsqdist := 0; {doesn't matter, just to prevent warnings}
    For I := 1 To Gr[B, 0] Do
    Begin
      If Dir(A, B, Gr[B, I]) > 0 Then
      Begin
        newdp := DotProduct(Norm^[A, B], Norm^[B, Gr[B, I]]);
        newsqdist := SqDist(B, Gr[B, I]);
        If (newdp < bestdp) Or
           ((newdp = bestdp) And (newsqdist < bestsqdist)) Then
        Begin
          bestdp := newdp;
          bestsqdist := newsqdist;
          bestp := Gr[B, I];
        End;
      End;
    End;
    Trace := bestp;
  End
  Else
  Begin
    bestsqdist := $7FFFFFFF;
    For I := 1 To Gr[B, 0] Do
    Begin
      If Dir(A, B, Gr[B, I]) = 0 Then
      Begin
        newdp := DotProduct(Norm^[A, B], Norm^[B, Gr[B, I]]);
        If newdp > 0.9 Then
        Begin
          newsqdist := SqDist(B, Gr[B, I]);
          If newsqdist < bestsqdist Then
          Begin
            bestsqdist := newsqdist;
            bestp := Gr[B, I];
          End;
        End;
      End;
    End;
    If bestsqdist <> $7FFFFFFF Then
      Exit(bestp);

    bestdp := -200;
    For I := 1 To Gr[B, 0] Do
    Begin
      If Dir(A, B, Gr[B, I]) < 0 Then
      Begin
        newdp := DotProduct(Norm^[A, B], Norm^[B, Gr[B, I]]);
        newsqdist := SqDist(B, Gr[B, I]);
        If (newdp > bestdp) Or
           ((newdp = bestdp) And (newsqdist < bestsqdist)) Then
        Begin
          bestdp := newdp;
          bestsqdist := newsqdist;
          bestp := Gr[B, I];
        End;
      End;
    End;
    If bestdp > -150 Then
      Exit(bestp);
    bestsqdist := $7FFFFFFF;
    For I := 1 To Gr[B, 0] Do
    Begin
      If Dir(A, B, Gr[B, I]) = 0 Then
      Begin
        newdp := DotProduct(Norm^[A, B], Norm^[B, Gr[B, I]]);
        If newdp < -0.9 Then
        Begin
          newsqdist := SqDist(B, Gr[B, I]);
          If newsqdist < bestsqdist Then
          Begin
            bestsqdist := newsqdist;
            bestp := Gr[B, I];
          End;
        End;
      End;
    End;
    Exit(bestp);
  End;
End;

Procedure BahurTrace(S1, S2 : Integer);

Var
  A, B, C : Integer;
  I : Integer;
  dir : Integer;
  p1, p2 : Integer;
  X1, Y1, X2, Y2 : Integer;

Begin
  A := S1;
  B := S2;
  PathLen := 0;
  If traced[A, B] Then
    Exit;
  Repeat
    Inc(PathLen);
    Path[PathLen] := A;
    C := Trace(A, B);
    traced[A, B] := True;
    A := B;
    B := C;
  Until (A = S1) And (B = S2);
  {validate layout}
  If PathLen <> K Then
    Exit;
  dir := 0;
  For I := 1 To PathLen Do
  Begin
    p1 := Path[I];
    If I < PathLen Then
      p2 := Path[I + 1]
    Else
      p2 := Path[1];
    X1 := Point[p1].X;
    Y1 := Point[p1].Y;
    X2 := Point[p2].X;
    Y2 := Point[p2].Y;
    Inc(dir, (X2 - X1) * (Y1 + Y2));
  End;
  If dir <= 0 Then
    Exit;
  FillChar(vis, SizeOf(Boolean) * (N + 5), 0);
  For I := 1 To PathLen Do
  Begin
    If vis[Path[I]] Then
      Exit;
    vis[Path[I]] := True;
  End;
  Inc(Res);
End;

Procedure Bahur;

Var
  I, J : Integer;

Begin
  Res := 0;
  For I := 1 To N Do
    For J := 1 To Gr[I, 0] Do
      Precalc_Norm(I, Gr[I, J]);
  For I := 1 To N Do
    For J := 1 To Gr[I, 0] Do
    Begin
{      Writeln(I, ' ', Gr[I, J], ' ', Trace(I, Gr[I, J]));}
      BahurTrace(I, Gr[I, J]);
    End;
End;

Begin
{  Writeln('al');}
  New(Norm);
  ReadIn;
  Bahur;
  WriteOut;
{  Writeln(Res);}
{  Writeln(Dir(10, 12, 13));}
{  Writeln('/al');}
End.
